/* Filename: config.c
   Purpose:  Configuration File routines
   Compiler: wxDevCpp Version 7.3.1.3 (MINGW32)
   Project:  OutGauge Bus
   Original Author:   Vladimir Kadlec http://www.fi.muni.cz/~xkadlec
   
   Revision: Serial Interface and Packet Update revisions  - Pete Willard
   Version: LFS.Z28.01
          
 Revised: Pete Willard
          petewillard@gmail.com
          Non-applicable code removed        
   
   Routines:
            init_config       Preload default values
            print_config      Display current config values
            read_line         read a line from the config file
            read_config       read config and load settings into config struct

*/

/*
Gear Indicator is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Gear Indicator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Gear Indicator; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "log.h"
#include "config.h"

#define MAX_IP_SIZE 100




//------------------------------------------------------------------------------
// Define default settings if config file is missing
//
void init_config (Config *conf) {

    conf->ip = calloc (MAX_IP_SIZE, 1);               // IP ADDRESS                         
    strcpy (conf->ip, "127.0.0.1");                   
    conf->port = 30001;                               // UDP Port
    conf->com  = 2;                                   // COM Port # 
    conf->speed = 19200;
    conf->debug = 1;                                  // Debug Mode
    conf->fuel_warning = 5;
    conf->rpm_leds = 8;                               // # of LED indicators
    conf->debug = 1;                                  // Debug Enabled
}



//------------------------------------------------------------------------------
void print_config (Config *conf) {

    printf ("\nCurrent configuration:\n");
    printf ("======================\n");
    printf ("IP: \t%s \n",conf->ip);    // TCP/IP - normally localhost
    printf ("Port:\t%d \n",conf->port); // UDP Port 
    printf ("COM:\t%d \n", conf->com); // Com Port 
    printf ("Speed:\t%d \n", conf->speed); // Com Port           
    printf ("Fuel warning:\t%d%%\n",conf->fuel_warning);
    printf ("RPM LEDS:\t%d\n",conf->rpm_leds);       
    printf ("Debug Mode:\t%d \n", conf->debug);
}


//------------------------------------------------------------------------------
void read_line (char *file_buf, long *file_pos, char **line_buf)
{
    long i;
    *line_buf = file_buf + (*file_pos);
    i = *file_pos; 
    while ((file_buf[i] != '\n') && (file_buf[i] != 0))
        i++;
    file_buf[i] = 0;
    *file_pos = i + 1;
}





//------------------------------------------------------------------------------
int read_config (char *file_name, Config *conf) {
 FILE *f;   
char *file_buf, *line_buf, *wrd;
long file_size, file_pos = 0, line_no = 0;    
int value;
char *error_value;
//word lfs_flags;

//------------------------------------------
if ((f = fopen (file_name, "r")) == NULL) {
        error ("Can't open file: %s.\n", file_name);
        return (0);
}

fseek (f, 0, SEEK_END);
file_size = ftell (f);
fseek (f, 0, SEEK_SET);
if ((file_buf = (char *) calloc (file_size+1, 1)) == NULL) {
   error ("Can't allocate memory for config file.\n");
   return (0);
}
file_size = fread (file_buf, 1, file_size, f);
fclose (f);
    
while (file_pos < file_size) {
          
   read_line (file_buf, &file_pos, &line_buf);
   line_no++;
   // use the tokenizer to retrieve config options
   wrd = strtok (line_buf, " \t\n\r");
   for (; 1; wrd = strtok (NULL, " \t\n\r")) {
       if (wrd == NULL)
           break;
       if (wrd[0] == 0)
           continue;
       if (wrd[0] == '#')
           break;
            
            
       //-----[ PORT ]-----------------------------------------------------               
       if (strcasecmp (wrd, "Port") == 0) {  // If Token Matches then             
          wrd = strtok (NULL, " \t\n\r");    // Grab Next Token
          value = strtol(wrd, &error_value, 10);// Convert token to base 10
             if ((*error_value) != 0) {         // Yell if conversion fails
                warning("invalid port number\n");
                }                              
          conf->port = value;                // Otherwise, we are good                         
       }    
       //-----[ IP ]-------------------------------------------------------                
       if (strcasecmp (wrd, "IP") == 0) {
          wrd = strtok (NULL, " \t\n\r");
          strncpy (conf->ip, wrd, MAX_IP_SIZE); // in this case a string
          }                                     // is being copied              
       //-----[ COM ]-----------------------------------------------------                
       if (strcasecmp (wrd, "Com") == 0) {
          wrd = strtok (NULL, " \t\n\r");
          value = strtol(wrd, &error_value, 10);
                if ((*error_value) != 0) {
                   warning("invalid com port number\n");
                   }
          conf->com = value;                             
       }                                                     
            
       //-----[ SPEED ]---------------------------------------------------                
       if (strcasecmp (wrd, "Speed") == 0) {
          wrd = strtok (NULL, " \t\n\r");
          value = strtol(wrd, &error_value, 10);
                if ((*error_value) != 0) {
                   warning("invalid com speed number\n");
                   }
          conf->speed = value;                             
       }                                                     
            
                
           //-----[ DEBUG ]-----------------------------------------------------      
           if (strcasecmp (wrd, "Debug") == 0) {
              wrd = strtok (NULL, " \t\n\r");
              value = strtol(wrd, &error_value, 10);
                    if ((*error_value) != 0) {
                       warning("invalid DEBUG setting\n");
                       }
              conf->debug = value;                                
           }                               
                
               
           //-----[ Fuel_warning ]----------------------------------------------  
           if (strcasecmp (wrd, "Fuel_warning") == 0) {
              wrd = strtok (NULL, " \t\n\r");
              value = strtol(wrd, &error_value, 10);
                    if ((*error_value) != 0) {
                       warning("invalid blink rate\n");
                       }
              conf->fuel_warning = value;                                
           }                                      
 
   //-----[ RPM_leds ]----------------------------------------------  
           if (strcasecmp (wrd, "Rpm_leds") == 0) {
              wrd = strtok (NULL, " \t\n\r");
              value = strtol(wrd, &error_value, 10);
                    if ((*error_value) != 0) {
                       warning("invalid RPM Indicator value\n");
                       }
              conf->rpm_leds = value;                                
           }                                
 
  } // End For Loop
 } // End While Loop
free (file_buf);
return (1);
}    


